/*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XTest.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <locale.h>
#include <string.h>
#include <ctype.h>

#ifdef ICONV
#include <iconv.h>
#endif
static char keys[] = "f,wltau\\bqrkvyjgzhcne]dmspi[ox";

int main(int argc, char **argv) {
    Display* display;		
    int screen_num;
    char *display_name = getenv("DISPLAY");	
    Window window;   
    int revert_to, i, c;
    XKeyEvent event;
    
    if(argc < 1) {
        fprintf(stderr, "Usage: %s \"string\"\n%s", argv[0], argv[0]);
        return -1;
    }
    
    display = XOpenDisplay(display_name);
    
    if(!setlocale(LC_CTYPE, "") || !XSupportsLocale()) {
         fprintf(stderr, "error: no locale support\n");
         return -1;
    }
    
    if (display == NULL) {
        fprintf(stderr, "%s: Cannot open display: %s\n", argv[0], display_name);
        return -1;
    }

    screen_num = DefaultScreen(display);
//    XSetInputFocus(display, PointerRoot, RevertToPointerRoot, CurrentTime);
    XSync(display, False);

    XGetInputFocus(display, &window, &revert_to);    // get window with focus
    if (window == None) {
        fprintf(stderr, "error: No window is focused\n");
        return 0;
    }
            
    event.display = display;
    event.window = window;
    event.root = RootWindow(event.display, DefaultScreen(event.display));
    event.subwindow = None;
    event.time = CurrentTime;

    event.x = event.y = event.x_root = event.y_root = 0;
    event.same_screen = True;
    
    
    for(i = 0; i < strlen(argv[1]); i++) {
        c = argv[1][i];        
        event.state = 0;

        if(c < 0) {
            event.state += 0x2000;

            if((c & 0x20) > 0) {
                c ^= 0x20;
                event.state ++;
            } 
            c = keys[63 + c];

        } else if((c & 0x20) == 0) {
            event.state = 1;
            printf("%c\n", c);
            c |= 0x20;
        } else if((c > 32 && c < 42) || c == 58)
            event.state = 1;

        event.keycode =  XKeysymToKeycode(display, c);

        event.type = KeyPress;
        XSendEvent(event.display, event.window, True, KeyPressMask, (XEvent *)&event);
        
        event.type = KeyRelease;
        XSendEvent(event.display, event.window, True, KeyPressMask, (XEvent *)&event);
    }

    XSync(display, False);
    XFlush(display);    
    XCloseDisplay(display);    
    return 0;
}
